clear all
close all
clc

% If you want to reproduce each time the same output of normrnd, reset the 
% state of randn to the same fixed value each time you call normrnd. This
% can be useful when comparing results for different values of T.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The Delta Method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

T=100;

% monthly returns in percent
mu=1;
sigma=4;
SR=mu/sigma

X=normrnd(mu,sigma,T,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2.

mu_hat=mean(X);
sigma_hat=std(X);

SR_hat=mu_hat/sigma_hat

SR_hat_annualized=12/sqrt(12)*SR_hat

% mu_12 = 12 mu_1
% sigma_12 = sqrt(12) sigma_1
% => SR_12 = 12/sqrt(12) SR_1 = sqrt(12) SR_1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3.

VAR=1+.25*(mu_hat/sigma_hat)^2;

z=norminv(.975,0,VAR^.5);

delta_method=[SR_hat-z/sqrt(T) SR_hat+z/sqrt(T)]


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bootstrap Confidence Interval
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

B=2000;

indx_bootstrap=unidrnd(T,T,B);
X_star=X(indx_bootstrap);

SR_star=mean(X_star)./std(X_star);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2.

SR_star=sort(SR_star);

bootstrap=[2*SR_hat-SR_star(ceil(B*0.975)) 2*SR_hat-SR_star(floor(B*0.025))]

% The example is probably bad chosen given that the results of the two
% methods are very similar. It should at least convince you that the
% bootstrap method works. In principle, it is better to use the bootstrap
% method when the number of observations is limited.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The Block Bootstrap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%First,choose block size
k=10;%number of blocks
l=T/k;%length of the block
%Now, choose number of random samples of size T=100 from the original data
B1=2000; %set originally equal to same B of previous exercise

%Compute the block bootstrap using user-defined functions bootblock and block for the same
%Sharpe-Ratio function we have been considering so far
Block_SR_star=bootblock(X,k,l,B1);
Block_SR_star=sort(Block_SR_star);

%Using an approach similar to the previous exercise, we estimate the
%confidence intervals
BlockBootstrap=[2*SR_hat-Block_SR_star(ceil(B1*0.975)) 2*SR_hat-Block_SR_star(floor(B1*0.025))]

%%When computing the confidence intervals, we note that the Block Bootstrap
%%intervals tend to be wider than the traditional bootstrap (to see this, repeat the whole procedure several 
% times and you will see. 
%This means that the block bootstrap is usually not as accurate as the traditional
%%bootstrap. However, block-bootstrap allows us to deal with a degree of
%%dependency between observations not possible with the traditional
%%bootstrap.


